@@ -10,7 +10,7 @@ is.bound <- function(name) {
 
 # f(a,b) %::% A : B : C
 '%::%' <- function(signature, types) {
-  os <- options(keep.source=TRUE)
+  os <- options(keep.source=TRUE, keep.parse.data=TRUE)
   s.expr <- paste(deparse(substitute(signature)), collapse="\n")
   t.expr <- paste(deparse(substitute(types)), collapse="\n")
   text <- paste(s.expr,t.expr, sep=" %::% ")
@@ -36,7 +36,7 @@ is.bound <- function(name) {
   tree$signature <- paste(s.expr,"%::%",t.expr, sep=' ')
 
   add_type(name, tree)
-  options(keep.source=os$keep.source)
+  options(keep.source=os$keep.source, keep.parse.data=os$keep.parse.data)
   invisible()
 }
 
@@ -45,7 +45,7 @@ is.bound <- function(name) {
 # f(a,b) %when% { a < 0 } %as% { abs(a) + b }
 # f(a,b) %as% { a + b }
 '%as%' <- function(signature, body) {
-  os <- options(keep.source=TRUE)
+  os <- options(keep.source=TRUE, keep.parse.data=TRUE)
   s.expr <- paste(deparse(substitute(signature)), collapse="\n")
   b.expr <- paste(deparse(substitute(body)), collapse="\n")
   text <- paste(s.expr,b.expr, sep=" %as% ")
@@ -88,7 +88,7 @@ is.bound <- function(name) {
   tree$fill.defaults <- clean_defaults(tree)
 
   add_variant(name, tree, where)
-  options(keep.source=os$keep.source)
+  options(keep.source=os$keep.source, keep.parse.data=TRUE)
   invisible()
 }
 
@@ -143,6 +143,8 @@ NewObject <- function(type.fn,type.name, ...)
 # 0.238   0.000   0.238
 UseFunction <- function(fn,fn.name, ...)
 {
+  # If user has added more definitions, attempt to access it
+  fn <- tryCatch(get_lr(fn.name), error=function(e) fn)
   result <- NULL
   # u:0.007 s:0.002
   raw.args <- list(...)
@@ -737,7 +739,9 @@ body_fn <- function(raw.args, tree, where)
     arg.string <- ''
   else
     arg.string <- paste(raw.args$token, collapse=',')
-  fn.string <- sprintf("function(%s) { %s }", arg.string, body)
+  fn.string <- tidy_source(
+    text=sprintf("function(%s) { %s }", arg.string, body),
+    indent=2, output=FALSE)
   eval(parse(text=fn.string), where)
 }
 
@@ -1123,3 +1127,15 @@ parse_eval <- function(it, raw=NULL)
   invisible()
 }
 
+
+get_lr <- function(fn.name) {
+  fn <- NULL
+  frames <- sys.frames()
+  n <- length(frames)
+  while (n > 0) {
+    fn <- get0(fn.name, frames[[n]], inherits=FALSE)
+    if (! is.null(fn)) return(fn)
+    n <- n - 1
+  }
+  get(fn.name)
+}
